package com.ywt.alipaympapi.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.domain.AlipayTradeQueryModel;
import com.alipay.api.domain.MedicalNationalPayAuthInfo;
import com.alipay.api.request.AlipayCommerceMedicalAuthinfoAuthQueryRequest;
import com.alipay.api.request.AlipaySystemOauthTokenRequest;
import com.alipay.api.request.AlipayTradeAppPayRequest;
import com.alipay.api.response.AlipayCommerceMedicalAuthinfoAuthQueryResponse;
import com.alipay.api.response.AlipaySystemOauthTokenResponse;
import com.alipay.api.response.AlipayTradeQueryResponse;
import com.fasterxml.jackson.databind.JsonNode;
import com.ywt.alipaympapi.core.utils.*;
import com.ywt.alipaympapi.face.IPayFace;
import com.ywt.alipaympapi.face.impl.AlipayService;
import com.ywt.alipaympapi.models.Constants;
import com.ywt.alipaympapi.models.auth.AlipayMpCfg;
import com.ywt.alipaympapi.models.enums.HospitalNameEnum;
import com.ywt.alipaympapi.models.enums.OutpatientOrderPayInsTypeEnum;
import com.ywt.alipaympapi.models.enums.PaymentStatusEnum;
import com.ywt.alipaympapi.models.enums.SexEnum;
import com.ywt.alipaympapi.models.outpatient.*;
import com.ywt.core.exception.AppMessageException;
import com.ywt.gapi.Result;
import com.ywt.gapi.ResultCode;
import com.ywt.gapi.drug.GetOneKeyRegListRequest;
import com.ywt.gapi.drug.GetOneKeyRegListResponse;
import com.ywt.gapi.order.*;
import com.ywt.gapi.order.OutpatientOrder;
import com.ywt.gapi.ordercenter.*;
import com.ywt.gapi.taihe.outpaitent.*;
import com.ywt.gapi.taihe.outpaitent.BaseResult;
import com.ywt.gapi.third.taihe.*;
import com.ywt.gapi.third.taihe.GetOutpatientListRequest;
import com.ywt.gapi.third.taihe.GetOutpatientListResponse;
import com.ywt.gapi.wx_ins_pay.*;
import com.ywt.model.PageVO;
import com.ywt.model.Pagination;
import com.ywt.request.pay.*;
import com.ywt.response.pay.*;
import com.ywt.alipaympapi.service.IsvPayService;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import redis.clients.jedis.JedisCommands;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class IsvPayServiceImpl implements IsvPayService {
    @Resource
    private IPayFace payFace;

    public static final int TYPE_UNPAID_WX = 1;
    private static final int TYPE_PAID_WX = 2;

    public static final int TYPE_UNPAID_ZFB = 0;
    private static final int TYPE_PAID_ZFB = 1;

    private static final int TYPE_PAID = 2;
    // No pagination for now
    private static final int PAGE_INDEX = 1;
    private static final int PAGE_SIZE = 99999;


    @Autowired
    private TaiheServiceGrpc.TaiheServiceBlockingStub taiheServiceBlockingStub;

    @Autowired
    private TaiheOutpatientServiceGrpc.TaiheOutpatientServiceBlockingStub taiheOutpatientServiceBlockingStub;

    @Autowired
    private com.ywt.gapi.drug.DrugServiceGrpc.DrugServiceBlockingStub drugServiceBlockingStub;

    @Autowired
    private OrderServiceGrpc.OrderServiceBlockingStub orderServiceBlockingStub;
    @Autowired
    private OrderCenterServiceGrpc.OrderCenterServiceBlockingStub orderCenterServiceBlockingStub;

    @Autowired
    private WxInsPayServiceGrpc.WxInsPayServiceBlockingStub wxInsPayServiceBlockingStub;

    @Autowired
    JedisCommands jedisCommands;

    @Autowired
    AlipayService alipayService;


    // 商家订单号获取
    @Override
    public TradeNoResponseData queryTradeNo(OutTradeNoRequest request) throws Exception {
        int orderId = Integer.parseInt(request.getOrderId());
        if (orderId <= 0) {
            throw new AppMessageException("该订单不存在");
        }
        int userId = Integer.parseInt(request.getUserId());
        int amount = BizUtil.yuan2Cent(request.getAmount());

        CreateAlipayOrderResponse response = orderCenterServiceBlockingStub.createAlipayOrder(CreateAlipayOrderRequest.newBuilder()
                .setBody("预约挂号")
                .setAlipayUid(request.getAlipayUid())
                .setOrderId(orderId)
                .setOrderNo("")
                .setAmount(amount)
                .setUserId(userId)
                .setTerminal(ContextHelper.getCurrentTerminalWrapped())
                .build());
        if (response.getCode() != ResultCode.SUCCEED_VALUE)
            throw new AppMessageException("支付宝下单：" + response.getInfo());
        TradeNoResponseData data = new TradeNoResponseData();
        data.setOrderId(orderId);
        data.setOutTradeNo(response.getOrderNo());
        data.setTradeNo(response.getTradeNo());
        return data;
    }

    @Override
    public TradeResultResponseData informTradeResult(TradeResultRequest request) {
        TradeResultResponseData data = new TradeResultResponseData();

        AlipayTradeQueryModel alipayTradeQueryModel = new AlipayTradeQueryModel();
        alipayTradeQueryModel.setTradeNo(request.getTradeNo());
        AlipayTradeQueryResponse response = payFace.tradeQuery(request.getAppId(), alipayTradeQueryModel);
        if ("TRADE_SUCCESS".equals(response.getTradeStatus())) {
            data.setStatus("TRADE_SUCCESS");
        }

        return data;
    }

    @Override
    public PayDetailResponseData queryPayDetail(PayDetailRequest request) throws Exception {
        int userId = ContextHelper.getCurrentUserIdWrapped();
        int hospitalId = BizUtil.getCurrentHospitalId();
        if (Checker.isNone(request.getStatus())) {
            throw new RuntimeException("订单状态不能为空");
        }
        if (request.getStatus() == TYPE_UNPAID_ZFB) {
            PayDetailResponseData da = new PayDetailResponseData();
            da.setDeptName(Checker.getStringValue(request.getDeptName()));
            if (!Checker.isNone(request.getAge()) && request.getAge() > 0) {
                da.setAge(request.getAge());
            }
            da.setSex(Checker.getStringValue(request.getSex()));
            da.setStatus(request.getStatus());
            da.setDoctorName(Checker.getStringValue(request.getDoctorName()));
            da.setDoctorCode(Checker.getStringValue(request.getDoctorCode()));
            da.setHisClinicCode(Checker.getStringValue(request.getHisClinicCode()));
            da.setPrescribeDate(Checker.getStringValue(request.getPrescribeDate()));
            da.setTotal(request.getTotal());
            da.setUserName(Checker.getStringValue(request.getName()));
            da.setName(Checker.getStringValue(request.getName()));
            da.setHisOrderNo(Checker.getStringValue(request.getHisOrderNo()));
            da.setAmount(Checker.getStringValue(request.getAmount()));
            da.setCreateTime(Checker.getStringValue(request.getPrescribeDate()));
            da.setHisPatientId(request.getHisPatientId());
            if (!Checker.isNone(request.getOrderItems())) {
                da.setOrderItems((request.getOrderItems()));
            }
            return da;
        }
        if (Checker.isNone(request.getInvoiceNo()) && Checker.isNone(request.getOutTradeNo())) {
            throw new RuntimeException("订单发票号不存在");
        }
        List<OrderItem> list = new LinkedList<>();
        PayDetailResponseData da = new PayDetailResponseData();
        String hisPatientId = Checker.getStringValue(request.getHisPatientId());
        switch (hospitalId) {
            default:
            case Constants.HOSPITAL_ID_NFYYBYFY:
                // 白云从 HIS 接口获取门诊缴费详情
                String invoiceNo = Checker.getStringValue(request.getInvoiceNo());
                CheckUtil.ensureNotEmpty(invoiceNo, "发票号不能为空");
                CheckUtil.ensureNotEmpty(hisPatientId, "HIS 病人 id 不能为空");
                // 根据 his_patient_id 获取诊疗卡信息
                String pName = "";
                int pAge = 0;
                String pGender = "";
                String cardNum = "";
                GetMedicalCardInfoByPatientIdRequest gResp = GetMedicalCardInfoByPatientIdRequest.newBuilder()
                        .setHisPatientId(hisPatientId)
                        .setHospitalId(hospitalId)
                        .setUserId(userId)
                        .build();
                GetMedicalCardInfoByPatientIdResponse gmcResp = taiheOutpatientServiceBlockingStub.getMedicalCardInfoByPatientId(gResp);
                if (gmcResp.getCode() == ResultCode.SUCCEED_VALUE) {
                    pName = gmcResp.getPatientName();
                    pAge = getPatientAgeSafely(gmcResp.getIdNo(), gmcResp.getBirthday());
                    pGender = IdCardUtil.getSex(gmcResp.getIdNo());
                    cardNum = Checker.getStringValue(gmcResp.getCardNo());

                }
                GetPaymentDetailResponse response2 = taiheServiceBlockingStub.getPaymentDetail(GetPaymentDetailRequest.newBuilder()
                        .setHospitalId(hospitalId)
                        .setInvoiceNo(invoiceNo)
                        .setHisPatientId(hisPatientId)
                        .setType("2")
                        .build());
                if (response2.getCode() != ResultCode.SUCCEED_VALUE)
                    throw new RuntimeException(response2.getMsg());
                if (!Checker.isNone(response2.getDetailList())) {
                    list = response2.getDetailList().stream()
                            .map(i -> {
                                OrderItem it = new OrderItem();
                                it.setItemName(i.getItemName());
                                it.setAmount(String.format("%.2f", i.getItemPrice() / 100d));
                                it.setItemNum(String.valueOf(i.getItemNumber()));
                                return it;
                            })
                            .collect(Collectors.toList());
                }
                da.setAge(Checker.getIntegerValue(pAge));
                da.setUserName(BizUtil.maskedName(Checker.getStringValue(pName)));
                da.setSex(pGender);
                da.setPayDate(request.getDate());
                da.setOrderItems(list);
                da.setCardNum(cardNum);
                da.setOrderId("");
                da.setTotal(request.getTotal());
                da.setStatus(request.getStatus());
                da.setAmount(request.getAmount());
                da.setHisPatientId(request.getHisPatientId());
                da.setDoctorName(Checker.getStringValue(request.getDoctorName()));
                da.setDeptName(Checker.getStringValue(request.getDeptName()));
                da.setPayDate(Checker.getStringValue(request.getPayTime()));
                da.setPrescribeDate(Checker.getStringValue(request.getPrescribeDate()));
                break;
            case Constants.TAIHE_HOSPITAL_ID:
                // 太和从数据库读取数据
                String hisOrderNo = request.getHisOrderNo();
                CheckUtil.ensureNotEmpty(hisOrderNo, "hisOrderNo 不能为空");
                GetOutpatientOrderDetailRequest req = GetOutpatientOrderDetailRequest.newBuilder()
                        .setHospitalId(BizUtil.getHospitalIdByTerminalId(ContextHelper.getCurrentTerminalWrapped()))
                        .setHisOrderNo(hisOrderNo)
                        .build();
                GetOutpatientOrderDetailResponse response = taiheOutpatientServiceBlockingStub.getOutpatientOrderDetail(req);
                if (response.getCode() != ResultCode.SUCCEED_VALUE)
                    throw new AppMessageException(response.getMsg());
                if (!Checker.isNone(response.getDetailList())) {
                    list = response.getDetailList().stream()
                            .map(i -> {
                                OrderItem it = new OrderItem();
                                it.setItemName(i.getItemName());
                                it.setAmount(String.format("%.2f", i.getItemPrice() / 100d));
                                it.setItemNum(String.valueOf(i.getItemNumber()));
                                return it;
                            })
                            .collect(Collectors.toList());
                }
                da.setOrderItems(list);
                da.setAge(Checker.getIntegerValue(response.getPatientAge()));
                da.setUserName(BizUtil.maskedName(Checker.getStringValue(response.getPatientName())));
                da.setSex(SexEnum.getSexDisplayName(Checker.getIntegerValue(response.getPatientSex())));
                da.setPayDate(request.getDate());
                da.setOrderItems(list);
//                da.setCardNum(cardNum);
                da.setOrderId("");
                da.setTotal(request.getTotal());
                da.setStatus(request.getStatus());
                da.setAmount(request.getAmount());
                da.setHisPatientId(request.getHisPatientId());
                da.setDoctorName(Checker.getStringValue(request.getDoctorName()));
                da.setDeptName(Checker.getStringValue(request.getDeptName()));
                da.setPayDate(Checker.getStringValue(request.getPayTime()));
                da.setPrescribeDate(Checker.getStringValue(request.getPrescribeDate()));
                break;
        }
        return da;
    }

    @Override
    public PageVO<PayHistoryResponseData> queryPayHistory(PayHistoryRequest request) throws Exception {
        List<PayHistoryResponseData> list = new ArrayList<>();
        int pageIndex = request.getPageIndex();
        int pageSize = request.getPageSize();
        int total = 0;
        if (Checker.isNone(request.getStatus())) {
            throw new AppMessageException("订单状态不能为空");
        }
        int status = Integer.parseInt(Checker.getStringValue(request.getStatus()));
        if (Checker.isNone(request.getHisPatientId())) {
            throw new AppMessageException("hisPatientI不能为空");
        }
        String hisPatientId = Checker.getStringValue(request.getHisPatientId());
        int userId = ContextHelper.getCurrentUserIdWrapped();
        String pName = "";
        int pAge = 0;
        String pGender = "";
        int hospitalId = BizUtil.getCurrentHospitalId();
        GetMedicalCardInfoByPatientIdRequest gResp = GetMedicalCardInfoByPatientIdRequest.newBuilder()
                .setHisPatientId(hisPatientId)
                .setHospitalId(hospitalId)
                .setUserId(userId)
                .build();
        GetMedicalCardInfoByPatientIdResponse resp = taiheOutpatientServiceBlockingStub.getMedicalCardInfoByPatientId(gResp);
        if (resp.getCode() == ResultCode.SUCCEED_VALUE) {
            pName = resp.getPatientName();
            pAge = getPatientAgeSafely(resp.getIdNo(), resp.getBirthday());
            pGender = IdCardUtil.getSex(resp.getIdNo());
        }
        switch (status) {
            case TYPE_UNPAID_ZFB:
                // 待缴费列表从 HIS 获取数据
                GetOutpatientListRequest req = GetOutpatientListRequest.newBuilder()
                        .setPatientId(hisPatientId)
                        .setType(TYPE_UNPAID_WX)
                        .setHospitalId(hospitalId)
                        .build();
                GetOutpatientListResponse result = taiheServiceBlockingStub.getOutpatientList(req);

                if (result.getCode() != ResultCode.SUCCEED_VALUE) {
                    throw new RuntimeException(result.getMsg());
                }
                List<Map<String, Object>> lst = new LinkedList<>();
                Map<String, Object> map;

                List<PaymentItem> paymentItems = result.getPaymentList();
                if (!Checker.isNone(paymentItems)) {
                    // 过滤处方门诊缴费单
                    GetOneKeyRegListRequest getOneKeyRegListRequest = GetOneKeyRegListRequest.newBuilder()
                            .setUserId(userId)
                            .build();
                    GetOneKeyRegListResponse getOneKeyRegListResponse = drugServiceBlockingStub.getOneKeyRegList(getOneKeyRegListRequest);
                    List<String> clinicCodes = getOneKeyRegListResponse.getCode() == ResultCode.SUCCEED_VALUE ? getOneKeyRegListResponse.getClinicCodeList() : new LinkedList<>();
                    List<PaymentItem> payItems = paymentItems;
                    if (!clinicCodes.isEmpty()) {
                        payItems = paymentItems.stream()
                                .filter(paymentItem -> !clinicCodes.contains(paymentItem.getClinicCode()))
                                .collect(Collectors.toList());
                    }
                    total = payItems.size();
                    for (PaymentItem payItem : payItems) {
                        PayHistoryResponseData data = new PayHistoryResponseData();
                        data.setStatus(status);
                        data.setTitle("");
                        data.setHisOrderNo(payItem.getHisOrdNum());
                        data.setOrderId(payItem.getOrderId());
                        data.setDeptName(payItem.getDeptName());
                        data.setPrescribeDate(payItem.getPrescriptionDate());
                        data.setName(BizUtil.maskedName(resp.getPatientName()));
                        data.setTotal(payItem.getTotalFee());
                        data.setAmount(String.format("%.2f", payItem.getTotalFee() / 100d));
                        data.setTradeNo(""); //Todo
                        data.setOutTradeNo("");//Todo
                        data.setIdNum("");//Todo
                        data.setInvoiceNo(Checker.getStringValue(payItem.getInvoiceNo()));
                        data.setHisPatientId(hisPatientId);
                        data.setAge(Checker.getIntegerValue(pAge));
                        data.setSex(pGender);
                        data.setDoctorName(payItem.getDoctorName());
                        data.setDoctorCode(payItem.getDoctorCode());
                        data.setHisClinicCode(payItem.getClinicCode());
                        data.setOrderInsType(payItem.getOrderInsType());
                        List<PayDetailItem> detailItems = payItem.getItemListList();
                        List<OrderItem> ls = new ArrayList<>();
//                        String detailName = "";
                        for (PayDetailItem detailItem : detailItems) {
                            OrderItem it = new OrderItem();
                            it.setAmount(String.format("%.2f", detailItem.getItemTotalFee() / 100d));
                            it.setItemName(detailItem.getItemName());
                            it.setItemNum(detailItem.getItemNumber());
                            ls.add(it);
                        }
////                        detailName.substring(1);
//                        data.setDetailName(detailName);
                        data.setOrderItems(ls);
                        data.setType(0);
                        data.setMedInsFee(payItem.getMedInsFee());
                        data.setSelfFee(payItem.getSelfFee());
                        data.setPayName(payItem.getPayName());
                        data.setPatName(pName);
                        list.add(data);
//                        map.put("hisClinicCode", payItem.getClinicCode());
//                        map.put("hisOrderNo", payItem.getHisOrdNum());
//                        map.put("patientName", pName);
//                        map.put("orderInsType", payItem.getOrderInsType());
//                        map.put("patientAge", pAge);
//                        map.put("patientSex", pGender);
//                        map.put("doctorCode", payItem.getDoctorCode());

//                        map.put("detailItems", items);
//                        lst.add(map);
                        // 保存门诊缴费明细到数据库
                        SaveOutpatientOrderDetailsRequest sReq = SaveOutpatientOrderDetailsRequest.newBuilder()
                                .setHisOrderNo(payItem.getHisOrdNum())
                                .addAllDetail(payItem.getItemListList().stream()
                                        .map(i -> OutpatientOrderDetail.newBuilder()
                                                .setItemName(i.getItemName())
                                                .setItemPrice(i.getItemPrice())
                                                .setItemCode(i.getItemCode())
                                                .setItemNumber(i.getItemNumber())
                                                .setItemSpec(i.getItemSpec())
                                                .setItemUnit(i.getItemUnit())
                                                .setItemTotalFee(i.getItemTotalFee())
                                                .build()).collect(Collectors.toList()))
                                .build();
                        BaseResult baseResult = taiheOutpatientServiceBlockingStub.saveOutpatientOrderDetails(sReq);
                        if (baseResult.getCode() != ResultCode.SUCCEED_VALUE) {
                            throw new RuntimeException("保存门诊缴费明细失败");
//                            logger.error("保存门诊缴费明细失败，hisOrderNo : {}", payItem.getHisOrdNum());
                        }
                    }
                }
                break;
            case TYPE_PAID_ZFB:
                switch (hospitalId) {
                    default:
                    case Constants.HOSPITAL_ID_NFYYBYFY:
                        // v1.3.7 白云改为从 HIS 接口读取数据
                        String startDate = request.getStartTime();
                        String endDate = request.getEndTime();
                        Date start = DateUtil.parseDate(startDate, DateUtil.DADE_FROMAT_YMD);
                        Date end = DateUtil.parseDate(endDate, DateUtil.DADE_FROMAT_YMD);
                        if (Checker.isNone(start) || Checker.isNone(end)) {
                            Pagination pagination = new Pagination(pageIndex, pageSize, total);
                            PageVO<PayHistoryResponseData> res = new PageVO<>(pagination, list);
                            return res;
                        }
//                        CheckUtil.ensureNotNull(start, "请选择时间");
//                        CheckUtil.ensureNotNull(end, "请选择时间");
                        long day = 0;
                        day = DateUtil.getDaySub(request.getEndTime(), request.getStartTime());
                        if (day < -15) {
//                            throw new AppMessageException("请选择15天以内");
                            Pagination pagination = new Pagination(pageIndex, pageSize, total);
                            PageVO<PayHistoryResponseData> res = new PageVO<>(pagination, list);
                            return res;
                        }
                        GetOutpatientListRequest req3 = GetOutpatientListRequest.newBuilder()
                                .setPatientId(hisPatientId)
                                .setType(TYPE_PAID_WX)
                                .setHospitalId(hospitalId)
                                .setStartDate(startDate)
                                .setEndDate(endDate)
                                .build();
                        GetOutpatientListResponse res3 = taiheServiceBlockingStub.getOutpatientList(req3);
                        if (res3.getCode() != ResultCode.SUCCEED_VALUE) {
//                            throw new RuntimeException(res3.getMsg());
                            Pagination pagination = new Pagination(pageIndex, pageSize, total);
                            PageVO<PayHistoryResponseData> res = new PageVO<>(pagination, list);
                            return res;

                        }
                        total = res3.getPaymentList().size();
                        for (PaymentItem payItem : res3.getPaymentList()) {
                            PayHistoryResponseData data = new PayHistoryResponseData();
                            data.setStatus(status);
                            data.setTitle("");
                            data.setDeptName(payItem.getDeptName());
                            data.setPrescribeDate(payItem.getPrescriptionDate());
                            data.setName(BizUtil.maskedName(resp.getPatientName()));
                            data.setAmount(String.format("%.2f", payItem.getTotalFee() / 100d));
                            data.setTradeNo(""); //Todo
                            data.setOutTradeNo("");//Todo
                            data.setIdNum("");//Todo
                            data.setInvoiceNo(payItem.getInvoiceNo());
                            data.setHisPatientId(hisPatientId);
                            data.setPayTime(payItem.getPrescriptionDate());
                            data.setTotal(Checker.getIntegerValue(payItem.getTotalFee()));
                            data.setDoctorName(Checker.getStringValue(payItem.getDoctorName()));
                            data.setDoctorCode(Checker.getStringValue(payItem.getDoctorCode()));
                            data.setHisClinicCode(Checker.getStringValue(payItem.getClinicCode()));
                            data.setAge(Checker.getIntegerValue(pAge));
                            data.setOrderInsType(payItem.getOrderInsType());
                            data.setType(0);
                            list.add(data);
                        }
                        break;
                    case Constants.TAIHE_HOSPITAL_ID:
                        // 太和已缴费列表读取数据库
                        GetOutpatientOrderListRequest req2 = GetOutpatientOrderListRequest.newBuilder()
                                .setType(TYPE_PAID)
                                .setHisPatientId(hisPatientId)
                                .setPageIndex(PAGE_INDEX)
                                .setPageSize(PAGE_SIZE)
                                .setUserId(userId)
                                .setHospitalId(hospitalId)
                                .setStartDate("")
                                .setEndDate("")
                                .build();
                        GetOutpatientOrderListResponse res = orderServiceBlockingStub.getOutpatientOrderList(req2);
                        Result result2 = res.getResult();
                        if (result2.getCode() != ResultCode.SUCCEED) throw new AppMessageException(result2.getInfo());
                        total = res.getOrderList().size();
                        for (OutpatientOrder order : res.getOrderList()) {
                            PayHistoryResponseData data = new PayHistoryResponseData();
                            data.setStatus(status);
                            data.setTitle("");
                            data.setDeptName(order.getDeptName());
                            data.setPrescribeDate(DateUtil.convertTimestampToDateString(order.getPrescriptionDate()));
                            data.setName(BizUtil.maskedName(resp.getPatientName()));
                            data.setAmount(String.format("%.2f", order.getTotal() / 100d));
                            data.setTradeNo(""); //Todo
                            data.setOutTradeNo("");//Todo
                            data.setIdNum("");//Todo
                            data.setInvoiceNo(order.getInvoiceNo());
                            data.setHisPatientId(hisPatientId);
//                            data.setPayTime(payItem.getPrescriptionDate());
                            data.setTotal(Checker.getIntegerValue(order.getTotal()));
                            data.setDoctorName(Checker.getStringValue(order.getDoctorName()));
                            data.setDoctorCode(Checker.getStringValue(order.getDoctorCode()));
//                            data.setHisClinicCode(Checker.getStringValue(payItem.getClinicCode()));
                            data.setAge(Checker.getIntegerValue(pAge));
                            data.setType(0);
                            data.setHisOrderNo(Checker.getStringValue(order.getHisOrderNo()));
                            data.setOrderId(Checker.getIntegerValue(order.getOrderId()));
                            data.setSex(SexEnum.getSexDisplayName(Checker.getIntegerValue(order.getPatientSex())));
                            list.add(data);
                        }
                        break;
                }
                break;
        }
        Pagination pagination = new Pagination(pageIndex, pageSize, total);
        PageVO<PayHistoryResponseData> res = new PageVO<>(pagination, list);
        return res;
    }

    @Override
    public ComputeSelfPaidOrderResponseData computeSelfPaidOrder(ComputeSelfPaidOrderRequest request) {
        return null;
    }

    @Override
    public ComputeMedicarePayOrderResponseData computeMedicarePayOrder(ComputeMedicarePayOrderRequest request) {
        return null;
    }

    @Override
    public TradeNoForOrderResponseData tradeNoForOrder(TradeNoForOrderRequest request) throws Exception {
        TradeNoForOrderResponseData data = new TradeNoForOrderResponseData();
        int userId = ContextHelper.getCurrentUserIdWrapped();
        int hospitalId = BizUtil.getCurrentHospitalId();
        int terminal = ContextHelper.getCurrentTerminalWrapped();
        String deptName = Checker.getStringValue(request.getDeptName());
        String doctorName = Checker.getStringValue(request.getDoctorName());
        int totalFee = Checker.getIntegerValue(request.getTotalFee());
        int medInsFee = Checker.getIntegerValue(request.getMedInsFee());
        int selfFee = Checker.getIntegerValue(request.getSelfFee());
        String patientId = Checker.getStringValue(request.getHisPatientId());
        String hisClinicCode = Checker.getStringValue(request.getHisClinicCode());
        String hisOrdNum = Checker.getStringValue(request.getHisOrderNo());
        String payName = Checker.getStringValue(request.getPayName());
        String patName = Checker.getStringValue(request.getName());
        String invoiceNo = Checker.getStringValue(request.getInvoiceNo());
        String doctorCode = Checker.getStringValue(request.getDoctorCode());
        int age = Checker.getIntegerValue(request.getAge());
        Date pDate = DateUtil.parseDate(request.getPrescribeDate(), "yyyy-MM-dd HH:mm:ss");
        int sex = 1;
        if (request.getSex().equals("女")) {
            sex = 2;
        }

        String orderInsType = Checker.getStringValue(request.getOrderInsType());
        int payInsType = Checker.getIntegerValue(request.getPayInsType());
        String insuAdmDr = Checker.getStringValue(request.getInsuAdmDr());
        String insUploadFeeResp = Checker.getStringValue(request.getInsUploadFeeResp());

        if (pDate == null) throw new AppMessageException("处方日期有误");
        CheckUtil.ensureNotEmpty(invoiceNo, "发票号不能为空");
        CheckUtil.ensureNotEmpty(doctorCode, "医生编码不能为空");
        CheckUtil.ensureNotEmpty(orderInsType, "订单费用类型不能为空");
        if (totalFee <= 0) throw new AppMessageException("缴费总额有误");
        if (medInsFee + selfFee != totalFee)
            throw new AppMessageException("医保部分金额与自费部分金额总和不等于缴费总额");
        if (OutpatientOrderPayInsTypeEnum.valueOf(payInsType) == OutpatientOrderPayInsTypeEnum.MED_INSURANCE) {
            // 医保支付校验必传参数
            if (StringHelper.isNullOrEmpty(insUploadFeeResp)) {
                throw new AppMessageException("医保支付需要传入医保上传费用信息等参数");
            }
        }
        String orderNo;
        int orderId;
        CreateOutpatientOrderRequest req;
        Result result;
        switch (OutpatientOrderPayInsTypeEnum.valueOf(payInsType)) {
            default:
            case SELF:
                req = CreateOutpatientOrderRequest.newBuilder()
                        .setDeptCode("")
                        .setDeptName(deptName)
                        .setDoctorCode(doctorCode)
                        .setDoctorName(doctorName)
                        .setTotal(totalFee)
                        .setReimbursement(medInsFee)
                        .setIndividual(selfFee)
                        .setHisPatientId(patientId)
                        .setHisClinicCode(hisClinicCode)
                        .setHisOrderNo(hisOrdNum)
                        .setPayName(payName)
                        .setPrescriptionDate(pDate.getTime())
                        .setPatientName(patName)
                        .setPatientAge(age)
                        .setPatientSex(sex)
                        .setOrderInsType(orderInsType)
                        .setInvoiceNo(invoiceNo)
                        .setPayInsType(payInsType)
                        .setHospitalId(hospitalId)
                        .setTerminal(terminal)
                        .build();
                result = orderServiceBlockingStub.createOutpatientOrder(req);
                if (result.getCode() != ResultCode.SUCCEED) throw new AppMessageException(result.getInfo());
                orderNo = result.getValue();
                orderId = getOrderIdByOrderNo(orderNo);
                CreateAlipayOrderResponse response = orderCenterServiceBlockingStub.createAlipayOrder(CreateAlipayOrderRequest.newBuilder()
                        .setBody("门诊缴费")
                        .setAlipayUid(request.getAlipayUid())
                        .setOrderId(orderId)
                        .setOrderNo(orderNo)
                        .setAmount(request.getTotal())
                        .setUserId(userId)
                        .setTerminal(ContextHelper.getCurrentTerminalWrapped())
                        .build());
                if (response.getCode() != ResultCode.SUCCEED_VALUE) {
                    throw new AppMessageException(response.getInfo());
                }
                data.setNeedPay(true);//设置是否需要支付宝自费支付
                data.setOutTradeNo(orderNo);
                data.setTradeNo(response.getTradeNo());
                return data;
            case MED_INSURANCE:
                // 获取身份证
                GetMedicalCardInfoByPatientIdRequest gResp = GetMedicalCardInfoByPatientIdRequest.newBuilder()
                        .setHisPatientId(patientId)
                        .setHospitalId(hospitalId)
                        .setUserId(userId)
                        .build();
                GetMedicalCardInfoByPatientIdResponse resp = taiheOutpatientServiceBlockingStub.getMedicalCardInfoByPatientId(gResp);
                if (resp.getCode() != ResultCode.SUCCEED_VALUE) {
                    throw new AppMessageException(String.format("获取不到诊疗卡信息: %s", resp.getMsg()));
                }
                String userCardNo = resp.getIdNo();
                if (StringHelper.isNullOrEmpty(userCardNo))
                    throw new AppMessageException("用户诊疗卡身份证信息为空");
                // 医保支付，先生成系统订单，然后继续使用支付宝接口下单
                req = CreateOutpatientOrderRequest.newBuilder()
                        .setDeptCode("")
                        .setDeptName(deptName)
                        .setDoctorCode(doctorCode)
                        .setDoctorName(doctorName)
                        .setTotal(totalFee)
                        .setReimbursement(medInsFee)
                        .setIndividual(selfFee)
                        .setHisPatientId(patientId)
                        .setHisClinicCode(hisClinicCode)
                        .setHisOrderNo(hisOrdNum)
                        .setPayName(payName)
                        .setPrescriptionDate(pDate.getTime())
                        .setPatientName(patName)
                        .setPatientAge(age)
                        .setPatientSex(sex)
                        .setOrderInsType(orderInsType)
                        .setInvoiceNo(invoiceNo)
                        .setPayInsType(payInsType)
                        .setHospitalId(hospitalId)
                        .setInsuAdmDr(insuAdmDr)
                        .setInsUploadFeeResp(insUploadFeeResp)
                        .setTerminal(terminal)
                        .build();
                result = orderServiceBlockingStub.createOutpatientOrder(req);
                if (result.getCode() != ResultCode.SUCCEED) {
                    throw new AppMessageException(result.getInfo());
                }
                orderNo = result.getValue();
                orderId = getOrderIdByOrderNo(orderNo);
                String medicalCardId = "";
                String medicalCardInstId = "";
                String medOrgOrd = "";
                String payOrdId = "";
                String payAuthNo = "";
                JsonNode jn = JsonSerializer.readToNode(insUploadFeeResp);
                if (jn != null) {
                    medicalCardId = Checker.getStringValue(jn.get("medicalCardId").asText());
                    medicalCardInstId = Checker.getStringValue(jn.get("medicalCardInstId").asText());
                    payAuthNo = Checker.getStringValue(jn.get("payAuthNo").asText());
                    String output = Checker.getStringValue(jn.get("output").asText(), "{}");
                    JsonNode jn2 = JsonSerializer.readToNode(output);
                    if (jn2 != null) {
                        JsonNode jn3 = jn2.get("data");
                        if (jn3 != null) {
                            medOrgOrd = Checker.getStringValue(jn3.get("medOrgOrd").asText());
                            payOrdId = Checker.getStringValue(jn3.get("payOrdId").asText());
                        }
                    }
                }
                String subject = "门诊缴费";
                int total = request.getTotal();
                // 使用支付宝接口下单
                // 增加医保参数
                AlipayMpCfg cfg = BizUtil.getCfg(terminal);
                HashMap<String, String> bizContent = new HashMap<>();
                bizContent.put("out_trade_no", orderNo);
                bizContent.put("total_amount", String.format("%.2f", total / 100d));
                bizContent.put("subject", subject);
                bizContent.put("product_code", "QUICK_MSECURITY_PAY");

                JSONObject extendParams = new JSONObject();
                extendParams.put("sys_service_provider_id", cfg.getIsvPid());
                extendParams.put("IS_INSURANCE_PAY", "T"); //是否使用医保支付
                extendParams.put("medical_card_inst_id", medicalCardInstId); //医保电子凭证机构号 获取线上医保业务授权接口返回medicalCardInstId参数。
                extendParams.put("medical_card_id", medicalCardId); //医保电子凭证授权码    获取线上医保业务授权接口返回medicalCardId参数。
                extendParams.put("medical_request_content", "");//医保参数透传体，没有需传空对象串

                JSONObject medicalRequestExt = new JSONObject(); //医保扩展参数
                medicalRequestExt.put("gmt_out_create", DateUtil.convertTimestampToDateString(new Date().getTime()));//医疗机构下单时间
                medicalRequestExt.put("out_trade_no", medOrgOrd); //商户网站唯一订单号   医院订单号medOrgOrd
                medicalRequestExt.put("serial_no", payOrdId); //移动支付中心订单号payOrdId
                medicalRequestExt.put("bill_no", payOrdId); //移动支付中心订单号payOrdId
                medicalRequestExt.put("industry", "HOSPITAL"); //行业标识
                medicalRequestExt.put("org_no", cfg.getYbv2OrgCode()); //医药机构编号
                medicalRequestExt.put("org_name", HospitalNameEnum.getDisplayName(hospitalId)); //医药机构名称
                medicalRequestExt.put("pay_auth_no", payAuthNo); //移动支付中心返回授权编号    获取线上医保业务授权接口返回payAuthNo参数
                medicalRequestExt.put("chinfo", "gjydzf"); //渠道标识
                medicalRequestExt.put("scene", "TREATMENT"); //使用场景
                medicalRequestExt.put("insurance_pay_scene", "OUTPATIENT");//医保结算场景
                medicalRequestExt.put("payment_city_code", "440100");//医保结算地城市编码(国标）
                extendParams.put("medical_request_ext", JSONObject.toJSONString(medicalRequestExt));
                bizContent.put("extend_params", JSONObject.toJSONString(extendParams));

                CreateAlipayOrderResponse createPayInsOrderResponse = orderCenterServiceBlockingStub.createAlipayOrder(CreateAlipayOrderRequest.newBuilder()
                        .setBody(subject)
                        .setAlipayUid(request.getAlipayUid())
                        .setOrderId(orderId)
                        .setOrderNo(orderNo)
                        .setAmount(total)
                        .setUserId(userId)
                        .setTerminal(ContextHelper.getCurrentTerminalWrapped())
                        .putAllBizContent(bizContent)
                        .setUseAppPay(true)
                        .build());
                if (createPayInsOrderResponse.getCode() == ResultCode.SUCCEED_VALUE) {
                    data.setNeedPay(true);//设置是否需要支付宝自费支付
                    data.setOutTradeNo(orderNo);
                    data.setTradeNo(createPayInsOrderResponse.getTradeNo());
                    data.setOrderStr(createPayInsOrderResponse.getOrderStr());
                    return data;
                } else {
                    log.error("IsvPayServiceImpl#tradeNoForOrder(reqData={} ):支付宝下单接口返回失败：{}", request.toString(), createPayInsOrderResponse.getInfo());
                    MarkOpOrderDeletedResponse moodResp = wxInsPayServiceBlockingStub.markOpOrderDeleted(MarkOpOrderDeletedRequest.newBuilder()
                            .setOrderNo(orderNo)
                            .build());
                    if (moodResp.getCode() != ResultCode.SUCCEED_VALUE) {
                        log.error("IsvPayServiceImpl#tradeNoForOrder():markOpOrderDeleted\n{}", moodResp.getMsg());
                    }
                    throw new AppMessageException(createPayInsOrderResponse.getInfo());
                }
        }
    }


    /**
     * 根据身份证或出生日期获取年龄，避免当身份证为空的情况下获取年龄会返回奇怪的值
     *
     * @param idNo     身份证
     * @param birthday 出生年月日，格式 yyyy-MM-dd
     * @return
     */
    private int getPatientAgeSafely(String idNo, String birthday) {
        if (!Checker.isNone(idNo)) {
            return IdCardUtil.getCurrentAge(idNo);
        } else {
//            logger.info("DHOutpatientController#getPatientAgeSafely(birthday = {}): 身份证为空", birthday);
            return IdCardUtil.getCurrentAgeByBirthday(birthday, IdCardUtil.DATE_PATTERN2);
        }
    }

    @Deprecated
    private String prePayConfirm(String patientId, String clinicCode, String hisOrdNum, int hospitalId, int orderSum) throws AppMessageException {
        PrePayConfirmRequest req = PrePayConfirmRequest.newBuilder()
                .setCardNo("")
                .setPatientId(patientId)
                .setStartDate("")
                .setEndDate("")
                .setClinicCode(clinicCode)
                .setHisOrdNum(hisOrdNum)
                .setOrderSum(orderSum)
                .setHospitalId(hospitalId)
                .setPayMode(Constants.HIS_PAY_MODE_ALIPAY)
                .build();
        PrePayConfirmResponse response = taiheServiceBlockingStub.prePayConfirm(req);
        if (response.getCode() == ResultCode.SUCCEED_VALUE) {
            if (!Checker.isNone(response.getListList())) {
                // 预结算成功，自动取消之前生成的未支付订单，让前端可以重新创建订单
                autoCancelPrevOrder(hisOrdNum);
                PrePayConfirm p = response.getListList().get(0);
                return p.getInvoiceNo();
            }
            throw new AppMessageException("预结算：HIS 返回列表数据为空");
        }
        throw new AppMessageException(response.getMsg());
    }

    @Deprecated
    private void autoCancelPrevOrder(String hisOrderNo) throws AppMessageException {
        GetOutpatientOrderByHisOrderNoRequest request = GetOutpatientOrderByHisOrderNoRequest.newBuilder()
                .setHisOrderNo(hisOrderNo)
                .setHospitalId(BizUtil.getHospitalIdByTerminalId(ContextHelper.getCurrentTerminalWrapped()))
                .build();
        GetOutpatientOrderByHisOrderNoResponse response = taiheOutpatientServiceBlockingStub.getOutpatientOrderByHisOrderNo(request);
        if (response.getCode() == ResultCode.SUCCEED_VALUE) {
            String orderNo = response.getOrderNo();
            if (!StringHelper.isNullOrEmpty(orderNo)) {
//                logger.info("DHOutpatientController#autoCancelPrevOrder(): 预结算自动取消之前生成的订单 orderNo = {}", orderNo);
                MarkOpOrderDeletedResponse markOpOrderDeletedResponse = wxInsPayServiceBlockingStub.markOpOrderDeleted(
                        MarkOpOrderDeletedRequest.newBuilder()
                                .setOrderNo(orderNo)
                                .build());
                if (markOpOrderDeletedResponse.getCode() != ResultCode.SUCCEED_VALUE) {

                    throw new AppMessageException(markOpOrderDeletedResponse.getMsg());
                }
            }
        } else {
            throw new AppMessageException(response.getMsg());
        }
    }

    //判断是否已线下缴费
    public GetOutpatientListResponse getOutpatientList(String hisPatientId, int type, Integer hospitalId) {
        int hid = Checker.getIntegerValue(hospitalId) > 0 ? hospitalId : Constants.TAIHE_HOSPITAL_ID;
        GetOutpatientListRequest req = GetOutpatientListRequest.newBuilder()
                .setPatientId(hisPatientId)
                .setType(type)
                .setHospitalId(hid)
                .build();
        return taiheServiceBlockingStub.getOutpatientList(req);
    }

    public int getOrderIdByOrderNo(String orderNo) {
        GetOrderInfoRequest request = GetOrderInfoRequest.newBuilder().setOrderNo(orderNo).build();
        GetOrderInfeResponse getOrderInfeResponse = orderCenterServiceBlockingStub.getOrderInfo(request);
        return getOrderInfeResponse.getOrderId();
    }

    /**
     * 获取医保支付授权
     * <a href="https://opendocs.alipay.com/pre-open/02iqci#5.2.2%20%E6%8E%A5%E5%8F%A3%E8%AF%A6%E7%BB%86%E8%AF%B4%E6%98%8E">参考文档</a>
     */
    public GetPayAuthInfo getPayAuthInfo(String appId, String alipayUid, String authCode, String callUrl, String reqBizNo) throws AppMessageException, AlipayApiException {
        GetPayAuthInfo getPayAuthInfo = new GetPayAuthInfo();
        // 换取 token
        AlipaySystemOauthTokenRequest req = new AlipaySystemOauthTokenRequest();
        req.setGrantType("authorization_code");
        req.setCode(authCode); //auth_code
        // ${hisOrdNum}-${userId}-${xxxx}
        AlipaySystemOauthTokenResponse resp = alipayService.getAlipayClient(appId).execute(req);
        if (!resp.isSuccess()) {
            log.error("IsvPayServiceImpl#getPayAuthInfo(appId={} , alipayUid={} , authCode={} , callUrl={} ):无法获取token: {}", appId, alipayUid, authCode, callUrl, JSONObject.toJSONString(resp));
            throw new AppMessageException(String.format("无法获取 token: %s", resp.getMsg()));
        }
        String accessToken = resp.getAccessToken();
        int terminal = BizUtil.getAlipayMpCfgByAppId(appId).getTerminal();
        AlipayMpCfg cfg = BizUtil.getCfg(terminal);

        AlipayCommerceMedicalAuthinfoAuthQueryRequest request = new AlipayCommerceMedicalAuthinfoAuthQueryRequest();
        JSONObject bizContent = new JSONObject();
        bizContent.put("alipay_user_id", alipayUid);  //支付宝用户ID
        bizContent.put("req_biz_no", reqBizNo);   // 业务请求唯一流水号，请务必保持每次请求的唯一性，如果保持不变则幂等返回原payAuthNo。
        bizContent.put("org_app_id", cfg.getYbv2AppId());  //定点医药机构在移动支付中心申请的应用Id
        bizContent.put("ins_code", "NATHSA"); //医保电子凭证机构编码，默认NATHSA
        bizContent.put("org_code", cfg.getYbv2OrgCode()); //定点医药机构编码
        bizContent.put("org_chnl_crtf_code", cfg.getYbv2OrgChnlCrtfCode()); //机构渠道认证编码
        bizContent.put("ol_biz_type_code", "04107"); //线上业务类型编码：04107-医保移动支付业务 04106-线上身份核验业务
        bizContent.put("call_url", callUrl); //授权回跳地址
        // TODO: 亲情付场景
//            JSONObject extendParams = new JSONObject(); //扩展参数，非必填
//            extendParams.put("sys_service_provider_id", "2088511833207846"); //合作伙伴返佣pid
//            extendParams.put("patient_name", "张三"); //就诊人真实姓名
//            extendParams.put("patient_cert_no", "10102272999111111"); //就诊人证件号
//            extendParams.put("patient_cert_type", "01"); //就诊人证件类型
//            bizContent.put("extend_params", extendParams);

        request.setBizContent(JSONObject.toJSONString(bizContent));
        log.info("验收：获取线上医保授权凭证：入参：{}", JSONObject.toJSONString(request));
        AlipayCommerceMedicalAuthinfoAuthQueryResponse response = alipayService.getAlipayClient(appId).execute(request, accessToken); //accessToken为用户授权token
        log.info("验收：获取线上医保授权凭证：出参：{}", JSONObject.toJSONString(response));
        // 出参
        String authUrl = ""; // 当用户需要授权时，返回该参数，医药机构前端直接跳转该地址。
        String payAuthNo = ""; // 当用户已经授权支付时，直接返回该字段。线上支付授权编码。
        String authNo = ""; // 线上身份验证授权码，只能用户身份核验，不能用于支付业务。
        String medicalCardInstId = ""; // 医保电子凭证机构编号，默认位NATHSA，请保存，后续支付接口需要使用。
        String medicalCardId = ""; // 用户医保电子凭证授权码，请保存，后续支付接口需要使用。
        if (!response.isSuccess()) {
            log.error("IsvPayServiceImpl#getPayAuthInfo(): 错误返回：" + response.getSubMsg());
            throw new AppMessageException(response.getSubMsg());
        }
        MedicalNationalPayAuthInfo info = response.getData();
        if (info != null) {
            authUrl = Checker.getStringValue(info.getAuthUrl());
            payAuthNo = Checker.getStringValue(info.getPayAuthNo());
            authNo = Checker.getStringValue(info.getAuthNo());
            medicalCardInstId = Checker.getStringValue(info.getMedicalCardInstId());
            medicalCardId = Checker.getStringValue(info.getMedicalCardId());
        } else {
            log.error("IsvPayServiceImpl#getPayAuthInfo(): 接口无响应 {} - {}", response.getMsg(), response.getErrorMessage());
            throw new AppMessageException(response.getSubMsg());
        }
        getPayAuthInfo.setAuthUrl(authUrl);
        getPayAuthInfo.setPayAuthNo(payAuthNo);
        getPayAuthInfo.setAuthNo(authNo);
        getPayAuthInfo.setMedicalCardInstId(medicalCardInstId);
        getPayAuthInfo.setMedicalCardId(medicalCardId);

        return getPayAuthInfo;
    }
}
