package com.ywt.alipaympapi.web.controller;

import com.fasterxml.jackson.databind.JsonNode;
import com.ywt.alipaympapi.core.utils.*;
import com.ywt.alipaympapi.models.BaseResponse;
import com.ywt.alipaympapi.models.BaseResponse2;
import com.ywt.alipaympapi.models.auth.AlipayMpCfg;
import com.ywt.alipaympapi.models.enums.MedInsTypeEnum;
import com.ywt.alipaympapi.models.enums.MpTypeEnum;
import com.ywt.alipaympapi.models.enums.OutpatientOrderPayInsTypeEnum;
import com.ywt.alipaympapi.models.outpatient.*;
import com.ywt.alipaympapi.service.impl.IsvPayServiceImpl;
import com.ywt.core.exception.AppMessageException;
import com.ywt.gapi.ResultCode;
import com.ywt.gapi.taihe.outpaitent.*;
import com.ywt.gapi.third.taihe.*;
import com.ywt.gapi.wx_ins_pay.MarkOpOrderDeletedRequest;
import com.ywt.gapi.wx_ins_pay.MarkOpOrderDeletedResponse;
import com.ywt.gapi.wx_ins_pay.WxInsPayServiceGrpc;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author Walker
 * Created on 2023/11/1
 */
@Slf4j
@RestController("/outpatient")
@RequestMapping({"/outpatient"})
public class OutpatientController {

    @Autowired
    private TaiheServiceGrpc.TaiheServiceBlockingStub taiheServiceBlockingStub;

    @Autowired
    private TaiheOutpatientServiceGrpc.TaiheOutpatientServiceBlockingStub taiheOutpatientServiceBlockingStub;

    @Autowired
    private WxInsPayServiceGrpc.WxInsPayServiceBlockingStub wxInsPayServiceBlockingStub;

    @Autowired
    private IsvPayServiceImpl isvPayService;

    @RequestMapping(value = {"/prePayConfirm"}, method = RequestMethod.POST)
    public @ResponseBody
    BaseResponse2<PrePayConfirmResp> prePayConfirm(@RequestBody PrePayConfirmReq reqData) {
        BaseResponse2<PrePayConfirmResp> baseResponse = new BaseResponse2<>();
        log.info("预结算开始1:hisOrdNum={},", reqData.getHisOrdNum());
        try {
            String cardNo = Checker.getStringValue(reqData.getCardNo());
            String patientId = Checker.getStringValue(reqData.getPatientId());
            String startDate = Checker.getStringValue(reqData.getStartDate());
            String endDate = Checker.getStringValue(reqData.getEndDate());
            String clinicCode = Checker.getStringValue(reqData.getClinicCode());
            String hisOrdNum = Checker.getStringValue(reqData.getHisOrdNum());
//            String callUrl = Checker.getStringValue(reqData.getCallUrl());
            int orderSum = Checker.getIntegerValue(reqData.getOrderSum());
            int payInsType = Checker.getIntegerValue(reqData.getPayInsType());
            int terminal = ContextHelper.getCurrentTerminalWrapped();
            if (terminal <= 0) {
                terminal = Checker.getIntegerValue(reqData.getTerminal());
            }
            int hospitalId = BizUtil.getHospitalIdByTerminalId(terminal);
            int userId = ContextHelper.getCurrentUserIdWrapped();
            if (userId <= 0) {
                userId = Checker.getIntegerValue(reqData.getUserId());
            }
            String alipayUid = ContextHelper.getAlipayUidWrapped();
            if (StringHelper.isNullOrEmpty(alipayUid)) {
                alipayUid = Checker.getStringValue(reqData.getAlipayUid());
            }
            int consumeType = Checker.getIntegerValue(reqData.getConsumeType());
            CheckUtil.ensureNotEmpty(alipayUid, "无法获取支付宝用户信息");
            CheckUtil.ensureNotEmpty(patientId, "HIS 患者 id 不能为空");
            CheckUtil.ensureNotEmpty(clinicCode, "就诊流水号不能为空");
            CheckUtil.ensureNotEmpty(hisOrdNum, "HIS 订单号不能为空");
            CheckUtil.ensureLargerThanZero(orderSum, "订单金额需大于 0");
            CheckUtil.ensureLargerThanZero(userId, "无法获取用户信息");
            CheckUtil.ensureLargerThanZero(terminal, "无法获取终端信息");
            AlipayMpCfg cfg = BizUtil.getCfg(terminal);
            String appId = cfg.getAppId();
            CheckUtil.ensureNotEmpty(appId, "缺少 appId 参数");
            PrePayConfirmRequest req = PrePayConfirmRequest.newBuilder()
                    .setCardNo(cardNo)
                    .setPatientId(patientId)
                    .setStartDate(startDate)
                    .setEndDate(endDate)
                    .setClinicCode(clinicCode)
                    .setHisOrdNum(hisOrdNum)
                    .setOrderSum(orderSum)
                    .setHospitalId(hospitalId)
                    .build();
            PrePayConfirmResponse response = taiheServiceBlockingStub.prePayConfirm(req);
            log.info("预结算开始2:hisOrdNum={},普通预结算responseCode:{}", reqData.getHisOrdNum(),response.getCode());
            if (response.getCode() == ResultCode.SUCCEED_VALUE) {
                if (!Checker.isNone(response.getListList())) {
                    // 预结算成功，自动取消之前生成的未支付订单，让前端可以重新创建订单
                    autoCancelPrevOrder(hisOrdNum, terminal);
                    PrePayConfirm p = response.getListList().get(0);
                    PrePayConfirmResp resp = new PrePayConfirmResp();
                    String invoiceNo = p.getInvoiceNo();
                    resp.setInvoiceNo(invoiceNo);
                    resp.setInvoiceAmt(p.getInvoiceAmt());
                    resp.setInsuFlag(p.getInsuFlag());
                    resp.setOutInsuInfo(p.getOutInsuInfo());
                    resp.setHisInsuInfo(p.getHisInsuInfo());
                    resp.setInvoiceInsuAmt(p.getInvoiceInsuAmt());
                    resp.setInvoiceSelfAmt(p.getInvoiceSelfAmt());
                    resp.setInsuOutTradeNo(p.getInsuOutTradeNo());
                    // 日志纪录第三方医保分解上传明细
                    log.info("预结算：hisOrdNum={}, 第三方医保分解上传明细：insuOutTradeNo={}", hisOrdNum, p.getInsuOutTradeNo());
                    if (OutpatientOrderPayInsTypeEnum.valueOf(payInsType) == OutpatientOrderPayInsTypeEnum.MED_INSURANCE) {
                        String orderInsType = Checker.getStringValue(reqData.getOrderInsType());
                        CheckUtil.ensureNotEmpty(orderInsType, "订单费用类型不能为空");
                        // 获取身份证和姓名
                        GetMedicalCardInfoByPatientIdRequest gResp = GetMedicalCardInfoByPatientIdRequest.newBuilder()
                                .setHisPatientId(patientId)
                                .setHospitalId(hospitalId)
                                .setUserId(userId)
                                .build();
                        GetMedicalCardInfoByPatientIdResponse gmcResp = taiheOutpatientServiceBlockingStub.getMedicalCardInfoByPatientId(gResp);
                        if (gmcResp.getCode() != ResultCode.SUCCEED_VALUE) {
                            return baseResponse.failedWithAppError(String.format("获取不到诊疗卡信息: %s(%s)", gmcResp.getMsg(), patientId));
                        }
                        String idCard = gmcResp.getIdNo();
                        String name = gmcResp.getPatientName();
                        // 如果选择医保，需要调用 HIS 接口上传费用
                        // 根据新老版医保调用不同的费用上传接口
                        MedInsTypeEnum mite = MedInsTypeEnum.valueOf(getMedInsTypeByOrderInsType(orderInsType, hospitalId));
                        if (mite == MedInsTypeEnum.SZT2) {
                            String payAuthNo = Checker.getStringValue(reqData.getPayAuthNo()); // 当用户已经授权支付时，直接返回该字段。线上支付授权编码。
                            String authNo = Checker.getStringValue(reqData.getAuthNo()); // 线上身份验证授权码，只能用户身份核验，不能用于支付业务。
                            String medicalCardInstId = Checker.getStringValue(reqData.getMedicalCardInstId()); // 医保电子凭证机构编号，默认位NATHSA，请保存，后续支付接口需要使用。
                            String medicalCardId = Checker.getStringValue(reqData.getMedicalCardId()); // 用户医保电子凭证授权码，请保存，后续支付接口需要使用。
                            CheckUtil.ensureNotEmpty(payAuthNo, "payAuthNo 不能为空");
                            CheckUtil.ensureNotEmpty(authNo, "authNo 不能为空");
                            CheckUtil.ensureNotEmpty(medicalCardInstId, "medicalCardInstId 不能为空");
                            CheckUtil.ensureNotEmpty(medicalCardId, "medicalCardId 不能为空");
                            PreSettleResponse preSettleResponse = taiheServiceBlockingStub.preSettle(PreSettleRequest.newBuilder()
                                    .setHospitalId(hospitalId)
                                    .setInvPrtDr(invoiceNo)
                                    .setCardNo(idCard)
                                    .setPsnName(name)
                                    .setMpType(MpTypeEnum.ALIPAY.getValue())
                                    .setAliPayAuthNo(payAuthNo)
                                    .setAcctUsedFlag(consumeType > 0 ? 0 : 1)
                                    .build());
                            log.info("预结算开始3:hisOrdNum={},医保2.0 responseCode:{}", reqData.getHisOrdNum(),preSettleResponse.getCode());
                            if (preSettleResponse.getCode() != ResultCode.SUCCEED_VALUE) {
                                return baseResponse.failedWithAppError(String.format("新版医保上传费用：%s", preSettleResponse.getInfo()));
                            }
                            PreSettleResp psr = new PreSettleResp();
                            psr.setHiOutput(preSettleResponse.getHiOutput());
                            String output = preSettleResponse.getOutput();
                            psr.setOutput(output);
                            psr.setPayAuthNo(preSettleResponse.getPayAuthNo());
                            psr.setSetlLatlnt(preSettleResponse.getSetlLatlnt());
//                            psr.setAuthUrl(authUrl);
                            psr.setPayAuthNo(payAuthNo);
                            psr.setAuthNo(authNo);
                            psr.setMedicalCardInstId(medicalCardInstId);
                            psr.setMedicalCardId(medicalCardId);
                            resp.setInsUploadFeeResp(JsonSerializer.toJson(psr));
                            resp.setInsuAdmDr(""); // 暂无该字段
                            resp.setInsuAdmInfoDr("");
                            String insuDivId = "";
                            if (!StringHelper.isNullOrEmpty(output)) {
                                JsonNode jn = JsonSerializer.readToNode(output);
                                if (jn != null) {
                                    JsonNode jn2 = jn.get("data");
                                    if (jn2 != null) {
                                        insuDivId = jn2.get("insuDivId").asText();
                                    }
                                }
                            }
                            resp.setInsuDivId(insuDivId);
                        } else {
                            GetInsuOPDividePreResponse getInsuOPDividePreResponse = taiheServiceBlockingStub.getInsuOPDividePre(GetInsuOPDividePreRequest.newBuilder()
                                    .setHospitalId(hospitalId)
                                    .setIdNo(idCard)
                                    .setInvoiceNo(invoiceNo)
                                    .setClinicCode(clinicCode)
                                    .setOrderInsType(orderInsType)
                                    .build());
                            if (getInsuOPDividePreResponse.getCode() != ResultCode.SUCCEED_VALUE) {
                                return baseResponse.failedWithAppError(String.format("医保上传费用：%s", getInsuOPDividePreResponse.getInfo()));
                            }
                            GetInsuOPDividePreResp r = new GetInsuOPDividePreResp();
                            r.setPsnNo(getInsuOPDividePreResponse.getPsnNo());
                            r.setMedType(getInsuOPDividePreResponse.getMedType());
                            r.setMdtrtId(getInsuOPDividePreResponse.getMdtrtId());
                            r.setInsuType(getInsuOPDividePreResponse.getInsuType());
                            r.setInsuAdmDr(getInsuOPDividePreResponse.getInsuAdmDr());
                            r.setRecerSysCode(getInsuOPDividePreResponse.getRecerSysCode());
                            r.setFixmedinsCode(getInsuOPDividePreResponse.getFixmedinsCode());
                            r.setFixmedinsName(getInsuOPDividePreResponse.getFixmedinsName());
                            r.setInfver(getInsuOPDividePreResponse.getInfver());
                            r.setMdtrtareaAdmvs(getInsuOPDividePreResponse.getMdtrtareaAdmvs());
                            r.setInsuplcAdmdvs(getInsuOPDividePreResponse.getInsuplcAdmdvs());
                            r.setPsnSetlway(getInsuOPDividePreResponse.getPsnSetlway());
                            r.setAcctUsedFlag(getInsuOPDividePreResponse.getAcctUsedFlag());
                            r.setChrgBchNo(getInsuOPDividePreResponse.getChrgBchNo());
                            r.setMedfeeSumamt(getInsuOPDividePreResponse.getMedfeeSumamt());
                            r.setOpterType(getInsuOPDividePreResponse.getOpterType());
                            r.setOpter(getInsuOPDividePreResponse.getOpter());
                            r.setOpterName(getInsuOPDividePreResponse.getOpterName());
                            r.setInsuAdmInfoDr(getInsuOPDividePreResponse.getInsuAdmInfoDr());
                            r.setInsuTypeCode(getInsuOPDividePreResponse.getInsuTypeCode());
                            r.setSessionId(getInsuOPDividePreResponse.getSessionId());
                            r.setCurrentSessionId(getInsuOPDividePreResponse.getCurrentSessionId());
                            r.setInsuDividePreDr(getInsuOPDividePreResponse.getInsuDividePreDr());
                            r.setInsuAdmInfoJson(getInsuOPDividePreResponse.getInsuAdmInfoJson());

                            resp.setInsUploadFeeResp(JsonSerializer.toJson(r));
                            resp.setInsuAdmDr(getInsuOPDividePreResponse.getInsuAdmDr());
                            resp.setInsuAdmInfoDr(getInsuOPDividePreResponse.getInsuAdmInfoDr());
                        }
                    }
                    return baseResponse.succeed(resp);
                }
                return baseResponse.failed(BaseResponse2.APP_ERROR, "预结算：HIS 返回列表数据为空");
            }
            return baseResponse.failed(BaseResponse.APP_ERROR, response.getMsg());
        } catch (AppMessageException ame) {
            return baseResponse.failed(BaseResponse.PARAMETER_ERROR, ame.getMessage());
        } catch (
                Exception e) {
            log.error("OutpatientController#prePayConfirm(reqData={} ):\n {}", reqData, e.getMessage(), e);
            return baseResponse.error(e.getMessage());
        }
    }

    private void autoCancelPrevOrder(String hisOrderNo, int terminal) {
        GetOutpatientOrderByHisOrderNoRequest request = GetOutpatientOrderByHisOrderNoRequest.newBuilder()
                .setHisOrderNo(hisOrderNo)
                .setHospitalId(BizUtil.getHospitalIdByTerminalId(terminal))
                .build();
        GetOutpatientOrderByHisOrderNoResponse response = taiheOutpatientServiceBlockingStub.getOutpatientOrderByHisOrderNo(request);
        if (response.getCode() == ResultCode.SUCCEED_VALUE) {
            String orderNo = response.getOrderNo();
            if (!StringHelper.isNullOrEmpty(orderNo)) {
                log.info("OutpatientController#autoCancelPrevOrder(): 预结算自动取消之前生成的订单 orderNo = {}", orderNo);
                MarkOpOrderDeletedResponse markOpOrderDeletedResponse = wxInsPayServiceBlockingStub.markOpOrderDeleted(
                        MarkOpOrderDeletedRequest.newBuilder()
                                .setOrderNo(orderNo)
                                .build());
                if (markOpOrderDeletedResponse.getCode() != ResultCode.SUCCEED_VALUE) {
                    log.error("OutpatientController#autoCancelPrevOrder(): markOpOrderDeleted failed: {}", markOpOrderDeletedResponse.getMsg());
                }
            }
        } else {
            log.error("OutpatientController#autoCancelPrevOrder(hisOrderNo={} ):\n {}", hisOrderNo, response.getMsg());
        }
    }

    /**
     * 根据 HIS 返回的订单医保类型，获取系统定义的医保类型值
     *
     * @param orderInsType HIS 返回的订单医保类型字符串
     * @return 系统医保类型值，{@link MedInsTypeEnum}
     * Copied from rpc project
     */
    private int getMedInsTypeByOrderInsType(String orderInsType, int hospitalId) {
        GetMedInsTypeByOrderInsTypeResponse response = taiheServiceBlockingStub.getMedInsTypeByOrderInsType(GetMedInsTypeByOrderInsTypeRequest.newBuilder()
                .setOrderInsTypeStr(orderInsType)
                .setHospitalId(hospitalId)
                .build());
        if (response.getCode() != ResultCode.SUCCEED_VALUE) {
            log.error("OutpatientController#getMedInsTypeByOrderInsType(orderInsType={} ):\n {}", orderInsType, response.getInfo());
            return MedInsTypeEnum.UNKNOWN.getValue();
        }
        return response.getOrderInsType();
    }

    /**
     * 获取线上医保业务授权接口
     * <a href="https://opendocs.alipay.com/pre-open/02iqci#5.2.1%20%E8%AF%A6%E7%BB%86%E6%8A%80%E6%9C%AF%E6%B5%81%E7%A8%8B">支付宝文档</a>
     */
    @RequestMapping(value = {"/getPayAuthInfo"}, method = RequestMethod.POST)
    public @ResponseBody BaseResponse2<GetPayAuthInfo> getPayAuthInfo(@RequestBody GetPayInfoReq reqData) {
        BaseResponse2<GetPayAuthInfo> baseResponse = new BaseResponse2<>();
        try {
            String appId = Checker.getStringValue(reqData.getAppId());
            String alipayUid = Checker.getStringValue(reqData.getAlipayUid());
            String authCode = Checker.getStringValue(reqData.getAuthCode());
            String callUrl = Checker.getStringValue(reqData.getCallUrl());
            String reqBizNo = Checker.getStringValue(reqData.getReqBizNo());
            CheckUtil.ensureNotEmpty(appId, "appId 不能为空");
            CheckUtil.ensureNotEmpty(alipayUid, "alipayUid 不能为空");
            CheckUtil.ensureNotEmpty(authCode, "authCode 不能为空");
            CheckUtil.ensureNotEmpty(callUrl, "callUrl 不能为空");
            CheckUtil.ensureNotEmpty(reqBizNo, "reqBizNo 不能为空");
            GetPayAuthInfo info = isvPayService.getPayAuthInfo(appId, alipayUid, authCode, callUrl, reqBizNo);
            return baseResponse.succeed(info);
        } catch (AppMessageException appMessageException) {
            return baseResponse.error(appMessageException.getMessage());
        } catch (Exception e) {
            log.error("OutpatientController#getPayAuthInfo(): {}", e.getMessage(), e);
            return baseResponse.error(e.getMessage());
        }
    }
}
